local coveredBridge = ...

--------------------------------------------------------------------------
local COVERED_BRIDGE_V3_PATH_A0_COMP = {
	TypeName = "COVERED_BRIDGE_V3_PATH_A0_COMP",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "FlagTerrain", Type = "integer", Default = 2 ^ OBJECT_FLAG.TERRAIN:toNumber() },
		{ Name = "FlagWater", Type = "integer", Default = 2 ^ OBJECT_FLAG.WATER:toNumber() },
		{ Name = "CoveredBridgeEndLength", Type = "float", Default = 20.0 },
	}
}

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_A0_COMP:create()
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_A0_COMP:init()
        local FromPosition = self:getOwner():getGlobalPosition()
        coveredBridge:log("FromPosition = " .. tostring(FromPosition))
        local Delta = self:checkNearestBank({ FromPosition[1], self:checkWaterElevation(FromPosition), FromPosition[3] }, 1)

        coveredBridge:log("Delta A0 = " .. tostring(Delta))
        coveredBridge:log("FinalPosition = " .. tostring(self:getOwner():getGlobalPosition()))
        self:getOwner():setGlobalPosition(Delta)
        coveredBridge:log("FinalPosition = " .. tostring(self:getOwner():getGlobalPosition()))
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_A0_COMP:checkWaterElevation(_workerDestination)
    local raycastResult2 = {}
     
    -- Raycast from the building position Y - 100
    -- to building position Y + 100
    -- only on objects with a WATER flag
    local FromPosition2 = { _workerDestination[1], _workerDestination[2]+100, _workerDestination[3] }
    local ToPosition2 = { _workerDestination[1], _workerDestination[2]-100, _workerDestination[3] }
    coveredBridge:log("FromPosition2 = " .. tostring(FromPosition2) .. "ToPosition2 = " .. tostring(ToPosition2))
    self:getLevel():rayCast(FromPosition2, ToPosition2, raycastResult2, self.FlagWater)
    coveredBridge:log("final raycastResult2 = RigidBody = " .. tostring(raycastResult2["RigidBody"]) .. " Position = " .. tostring(raycastResult2["Position"]) .. " Normal = " .. tostring(raycastResult2["Normal"]))
    if raycastResult2["RigidBody"] == nil then
         coveredBridge:logWarning("coveredBridge: Bridge NOT on water")
         return _workerDestination[2]
    else
         coveredBridge:log("checkWaterElevation = " .. raycastResult2["Position"][2])
         return raycastResult2["Position"][2]
    end
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_A0_COMP:checkNearestBank(_workerDestination, _direction)
    coveredBridge:log("A0: _workerDestination = " .. tostring(_workerDestination) .. " _direction = " .. tostring(_direction))
    local raycastResult2 = {}
    coveredBridge:log("A0:self:getOwner():getGlobalOrientation() = " .. tostring(self:getOwner():getGlobalOrientation()) ) 
    --sinp = 2 * (q.w * q.y - q.z * q.x)     
    local q = self:getOwner():getGlobalOrientation()
    coveredBridge:log("A0:q = " .. tostring(q) ) 
    local sinp = 2 * (q[4] * q[2] - q[3] * q[1])
    local cosp = q[4] * q[4] - q[2] * q[2]
    coveredBridge:log("A0:sinp = " .. tostring(sinp) .. " cosp = " .. tostring(cosp) ) 

    -- Raycast from the building position + Y = 100
    -- to building position - y = 100
    -- only on objects with a TERRAIN flag
    local FromPosition2 = { _workerDestination[1], _workerDestination[2], _workerDestination[3] }
    local ToPosition2 = { _workerDestination[1] - _direction * sinp * 100, _workerDestination[2], _workerDestination[3] - _direction * cosp * 100 }
    coveredBridge:log("checkNearestBank: FromPosition2 = " .. tostring(FromPosition2) .. "ToPosition2 = " .. tostring(ToPosition2))
    self:getLevel():rayCast(FromPosition2, ToPosition2, raycastResult2, self.FlagTerrain)
    coveredBridge:log("A0: final raycastResult2 = RigidBody = " .. tostring(raycastResult2["RigidBody"]) .. " Position = " .. tostring(raycastResult2["Position"]) .. " Normal = " .. tostring(raycastResult2["Normal"]))
    if raycastResult2["RigidBody"] == nil then
         coveredBridge:logWarning("coveredBridge: Bridge NOT on water")
         return _workerDestination
    else
         local FinalPosition = { raycastResult2["Position"][1] - _direction * sinp * self.CoveredBridgeEndLength,
                                 raycastResult2["Position"][2],
                                 raycastResult2["Position"][3] - _direction * cosp * self.CoveredBridgeEndLength }   
         coveredBridge:log("A0: FinalPosition = " .. tostring(FinalPosition))
         return FinalPosition
    end
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_A0_COMP:update()
end

--------------------------------------------------------------------------
coveredBridge:registerClass(COVERED_BRIDGE_V3_PATH_A0_COMP)


--------------------------------------------------------------------------
local COVERED_BRIDGE_V3_PATH_B0_COMP = {
	TypeName = "COVERED_BRIDGE_V3_PATH_B0_COMP",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "FlagTerrain", Type = "integer", Default = 2 ^ OBJECT_FLAG.TERRAIN:toNumber() },
		{ Name = "FlagWater", Type = "integer", Default = 2 ^ OBJECT_FLAG.WATER:toNumber() },
		{ Name = "CoveredBridgeEndLength", Type = "float", Default = 20.0 },
	}
}

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_B0_COMP:create()
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_B0_COMP:init()
        local FromPosition = self:getOwner():getGlobalPosition()
        coveredBridge:log("FromPosition = " .. tostring(FromPosition))
        local Delta = self:checkNearestBank({ FromPosition[1], self:checkWaterElevation(FromPosition), FromPosition[3] }, -1)

        coveredBridge:log("FinalPosition = " .. tostring(self:getOwner():getGlobalPosition()))
        self:getOwner():setGlobalPosition(Delta)
        coveredBridge:log("FinalPosition = " .. tostring(self:getOwner():getGlobalPosition()))
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_B0_COMP:checkWaterElevation(_workerDestination)
    local raycastResult2 = {}
     
    -- Raycast from the building position + Y = 100
    -- to building position - y = 100
    -- only on objects with a WATER flag
    local FromPosition2 = { _workerDestination[1], _workerDestination[2]+100, _workerDestination[3] }
    local ToPosition2 = { _workerDestination[1], _workerDestination[2]-100, _workerDestination[3] }
    coveredBridge:log("FromPosition2 = " .. tostring(FromPosition2) .. "ToPosition2 = " .. tostring(ToPosition2))
    self:getLevel():rayCast(FromPosition2, ToPosition2, raycastResult2, self.FlagWater)
    coveredBridge:log("final raycastResult2 = RigidBody = " .. tostring(raycastResult2["RigidBody"]) .. " Position = " .. tostring(raycastResult2["Position"]) .. " Normal = " .. tostring(raycastResult2["Normal"]))
    if raycastResult2["RigidBody"] == nil then
         coveredBridge:logWarning("coveredBridge: Bridge NOT on water")
         return _workerDestination[2]
    else
         coveredBridge:log("checkWaterElevation = " .. raycastResult2["Position"][2])
         return raycastResult2["Position"][2]
    end
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_B0_COMP:checkNearestBank(_workerDestination, _direction)
    coveredBridge:log("B0: _workerDestination = " .. tostring(_workerDestination) .. " _direction = " .. tostring(_direction))
    local raycastResult2 = {}
    coveredBridge:log("B0:self:getOwner():getGlobalOrientation() = " .. tostring(self:getOwner():getGlobalOrientation()) ) 
    --sinp = 2 * (q.w * q.y - q.z * q.x)     
    local q = self:getOwner():getGlobalOrientation()
    coveredBridge:log("B0:q = " .. tostring(q) ) 
    local sinp = 2 * (q[4] * q[2] - q[3] * q[1])
    local cosp = q[4] * q[4] - q[2] * q[2]
    coveredBridge:log("B0:sinp = " .. tostring(sinp) .. " cosp = " .. tostring(cosp) ) 

    -- Raycast from the building position + Y = 100
    -- to building position - y = 100
    -- only on objects with a TERRAIN flag
    local FromPosition2 = { _workerDestination[1], _workerDestination[2], _workerDestination[3] }
    local ToPosition2 = { _workerDestination[1] - _direction * sinp * 100, _workerDestination[2], _workerDestination[3] - _direction * cosp * 100 }
    coveredBridge:log("checkNearestBank: FromPosition2 = " .. tostring(FromPosition2) .. "ToPosition2 = " .. tostring(ToPosition2))
    self:getLevel():rayCast(FromPosition2, ToPosition2, raycastResult2, self.FlagTerrain)
    coveredBridge:log("B0: final raycastResult2 = RigidBody = " .. tostring(raycastResult2["RigidBody"]) .. " Position = " .. tostring(raycastResult2["Position"]) .. " Normal = " .. tostring(raycastResult2["Normal"]))
    if raycastResult2["RigidBody"] == nil then
         coveredBridge:logWarning("coveredBridge: Bridge NOT on water")
         return _workerDestination
    else
         local FinalPosition = { raycastResult2["Position"][1] - _direction * sinp * self.CoveredBridgeEndLength,
                                 raycastResult2["Position"][2],
                                 raycastResult2["Position"][3] - _direction * cosp * self.CoveredBridgeEndLength }   
         coveredBridge:log("B0: FinalPosition = " .. tostring(FinalPosition))
         return FinalPosition
    end
end

--------------------------------------------------------------------------
function COVERED_BRIDGE_V3_PATH_B0_COMP:update()
end

--------------------------------------------------------------------------
coveredBridge:registerClass(COVERED_BRIDGE_V3_PATH_B0_COMP)
