local coveredBridge = ...

CoveredBridgePrefix = "COVERED_BRIDGE_V3"

-- Apply building asset processor
coveredBridge:registerAssetProcessor("models/coveredBridge.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
coveredBridge:registerAssetProcessor("models/coveredBridgeDecoration.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
coveredBridge:registerAssetProcessor("models/coveredBridgeGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
coveredBridge:registerAssetProcessor("models/coveredBridgeAddition.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

local myMonument = {
	Id = CoveredBridgePrefix,
	Name = CoveredBridgePrefix .. "_MONUMENT_NAME",
	Description = CoveredBridgePrefix .. "_MONUMENT_DESC",
	BuildingType = "MONUMENT",
	IsManuallyUnlocked = false,
	IsDestructible = true,
	IsEditable = true,
	IsClearTrees = true
}

if (foundation.getGameVersion == nil or version.cmp(foundation.getGameVersion(), "1.6") < 0) then
    coveredBridge:log("Hello World: game version 1.5")
    myMonument.DataType = "MONUMENT"
    myMonument.BuildingPartSetList = {
		{
			Name = CoveredBridgePrefix .. "_NAME",
	                BuildingPartList = {
		            CoveredBridgePrefix .. "_CORE_PART",
		            CoveredBridgePrefix .. "_HOUSE_PART",
                            CoveredBridgePrefix .. "_RED_PART",
                            CoveredBridgePrefix .. "_GREEN_A_PART",
		            CoveredBridgePrefix .. "_GREEN_B_PART",
                            CoveredBridgePrefix .. "_BLUE_PART",
		            CoveredBridgePrefix .. "_RED_WINE_PART",
		            CoveredBridgePrefix .. "_CARBON_PART",
		            CoveredBridgePrefix .. "_CENTER_COVERAGE_PART",
		            CoveredBridgePrefix .. "_START_END_COVERAGE_PART",
		            CoveredBridgePrefix .. "_GATE_ROOT_PART"
	                }
                }
    }
    _MoverStart = "BUILDING_PART_MOVER"
    _MoverCore = "BUILDING_PART_MOVER_BRIDGE"
    _ConstructorDataStart = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_START_PREFAB",
		FillerList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		BasementScale = 2
    }
    _ConstructorDataEnd =  {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = CoveredBridgePrefix .. "_END_PREFAB",
		FillerList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		BasementScale = 2
    }
    _CoveredBridgeTagBehaviorWork = "WORK_BEHAVIOR"
    _BridgeBuildingZone = { Polygon = polygon.createRectangle( {1, 1} ) } 
    _GateBuildingZone = { Polygon = polygon.createRectangle( {8, 4} ) } 
else
    coveredBridge:log("Hello World: game version " .. foundation.getGameVersion())
    myMonument.DataType = "BUILDING"
    myMonument.AssetCoreBuildingPart = CoveredBridgePrefix .. "_CORE_PART"
    myMonument.BuildingPartSetList = {
		{
			Name = CoveredBridgePrefix .. "_NAME",
	                BuildingPartList = {
		            CoveredBridgePrefix .. "_HOUSE_PART",
                            CoveredBridgePrefix .. "_RED_PART",
                            CoveredBridgePrefix .. "_GREEN_A_PART",
		            CoveredBridgePrefix .. "_GREEN_B_PART",
                            CoveredBridgePrefix .. "_BLUE_PART",
		            CoveredBridgePrefix .. "_RED_WINE_PART",
		            CoveredBridgePrefix .. "_CARBON_PART",
		            CoveredBridgePrefix .. "_CENTER_COVERAGE_PART",
		            CoveredBridgePrefix .. "_START_END_COVERAGE_PART",
		            CoveredBridgePrefix .. "_GATE_ROOT_PART"
	                }
                }
    }
    _MoverStart = "BUILDING_PART_MOVER_BRIDGE"
    _MoverCore = "BUILDING_PART_MOVER"
    _ConstructorDataStart =  {
		DataType = "BUILDING_CONSTRUCTOR_SLOPE",
		CoreObjectPrefab = CoveredBridgePrefix .. "_START_PREFAB",
		SlopePartList = {
			CoveredBridgePrefix .. "_SLOPE_START_PART"
		},
		CoreBasementPartList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		UpperAttachType = ATTACH_NODE_TYPE.RIGHT,
		LowerAttachType = ATTACH_NODE_TYPE.LEFT
    }
    _ConstructorDataEnd = {
		DataType = "BUILDING_CONSTRUCTOR_SLOPE",
		CoreObjectPrefab = CoveredBridgePrefix .. "_END_PREFAB",
		SlopePartList = {
			CoveredBridgePrefix .. "_SLOPE_END_PART"
		},
		CoreBasementPartList = {
			CoveredBridgePrefix .. "_TILING_PART"
		},
		UpperAttachType = ATTACH_NODE_TYPE.LEFT,
		LowerAttachType = ATTACH_NODE_TYPE.RIGHT,
    } 
    _CoveredBridgeTagBehaviorWork = "BEHAVIOR_WORK"

    _BridgeBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {1, 1} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
    }}
    _GateBuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {8, 4} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
    }}
    coveredBridge:registerEnumValue ("BUILDING_PART_TYPE", "DECORATION")
end

-- Register all covered bridge building parts and their properties
coveredBridge:dofile("scripts/building_parts.lua")

-- Register the covered bridge monument
coveredBridge:register(myMonument)

function checkVersion(_name, _id, _minimalVersion)
    if (foundation.getModVersion == nil) then
        coveredBridge:log("Unknown mod version")
    elseif not foundation.isModEnabled(_id) then
        coveredBridge:msgBox(_name .. " disabled")
    elseif not foundation.isModLoade(_id) then
        coveredBridge:msgBox(_name .. " to be up in mod list")
    else
        local _myVersion = foundation.getModVersion(_id)
        coveredBridge:log("Uses " .. _name .. " Current version : " .. _myVersion .. " Minimum version : " .. _minimalVersion)
        if (version.cmp(_myVersion, _minimalVersion) < 0) then
            coveredBridge:logError("Please, load the last version of " .. _name)
        else
            return true
        end
    end
    return false
end

if (checkVersion("Market Addition V2", "c6d941ce-a02d-41c5-a57f-9a2d9d1b2a02", "2.0.0")) then
    MarketAdditionPrefix = "MARKET_ADDITION_V2"
    coveredBridge:dofile("scripts/addHatMaker.lua")
    coveredBridge:dofile("scripts/addKnifeGrinder.lua")
end